import sys
import os
from PyQt6.QtWidgets import QApplication, QMainWindow, QMessageBox, QFileDialog, QDialog
from PyQt6.QtCore import pyqtSlot, QDir, Qt
import numpy as np
import pyqtgraph as pg

from lifescale.gui.MainWindow import Ui_MainWindow
from lifescale.gui.dialog_about import Ui_Dialog_about
from lifescale.models.ls_data import LSData
from lifescale.gui.panel_metadata_model import PanelMetadataModel
from lifescale.gui.sample_summary_model import SampleSummaryModel
from lifescale.gui.masses_model import MassesModel
from lifescale.gui import resources
from lifescale import __version__, __author__, __git_repo__, __email__, __copyright__, __pypi_repo__

# https://numpy.org/doc/stable/reference/generated/numpy.histogram_bin_edges.html#numpy.histogram_bin_edges
NUMPY_HISTOGRAM_BIN_EDGES_OPTIONS = {
    'auto': 'Maximum of the ‘sturges’ and ‘fd’ estimators. Provides good all around performance.',
    'fd': 'Freedman Diaconis Estimator: Robust (resilient to outliers) estimator that takes into account data variability and data size.',
    'doane': 'An improved version of Sturges’ estimator that works better with non-normal datasets.',
    'scott': 'Estimator based on leave-one-out cross-validation estimate of the integrated squared error. Can be regarded as a generalization of Scott’s rule.',
    'rice': 'Estimator does not take variability into account, only data size. Commonly overestimates number of bins required.',
    'sturges': 'R’s default method, only accounts for data size. Only optimal for gaussian data and underestimates number of bins for large non-gaussian datasets.',
    'sqrt': 'Square root (of data size) estimator, used by Excel and other programs for its speed and simplicity.',
    'Num. of bins': 'User defined number of bins (max. 1000).'
}


class MainWindow(QMainWindow, Ui_MainWindow):
    """Main Window of the application."""

    def __init__(self):
        """Initializer."""

        # Attributes:
        # - GUI objects:
        self.sample_list = None
        self.glw_sample_histogram = None
        self.sample_histogram = None
        self.masses_region = None
        self.hist_marker_mean_all = None
        self.hist_marker_mean_selected = None
        self.hist_marker_median_selected = None
        self.hist_marker_median_all = None
        self.hist = None
        # - other objects:
        self.ls_data = None
        self.settings = {
            'hist_number_of_bins': None,
            'hist_bin_mode': 'auto',
        }

        # GUI:
        super().__init__()
        self.setupUi(self)

        # Connect signals and slots:
        self.action_Exit.triggered.connect(self.exit_application)
        self.actionLoad_xlsm_file.triggered.connect(self.on_menu_file_load_xlsm_file)
        self.action_About.triggered.connect(self.on_menu_help_about)
        self.action_csvFiles.triggered.connect(self.on_action_csvFiles)
        self.listWidget_Samples.itemSelectionChanged.connect(self.on_selection_changed_sample_list)
        self.checkBox_MassesShowTable.stateChanged.connect(self.on_checkBox_MassesShowTable_stateChanged)
        self.comboBox_histMethod.currentIndexChanged.connect(self.on_histogram_bin_method_currentIndexChanged)
        self.checkBox_plotMedian.stateChanged.connect(self.on_checkBox_plotMedian_stateChanged)
        self.checkBox_plotMean.stateChanged.connect(self.on_checkBox_plotMean_stateChanged)
        self.comboBox_histMethod.currentIndexChanged.connect(self.on_selection_changed_sample_list)
        self.spinBox_histBins.valueChanged.connect(self.on_selection_changed_sample_list)

        # Set up GUI items and widgets:
        self.status = self.statusBar()
        self.set_up_sample_histogram()
        self.set_up_sample_list()
        self.set_up_hist_method_comboBox()

        # GUI dialogs:
        self.dlg_about = DialogAbout()
        self.dlg_about.label_version.setText(__version__)
        self.dlg_about.label_git_repo.setText(__git_repo__)
        self.dlg_about.label_pypi_repo.setText(__pypi_repo__)
        self.dlg_about.label_email.setText(__email__)
        self.dlg_about.label_copyright.setText(__copyright__)
        self.dlg_about.label_author.setText(__author__)

        # Init models:
        self.panel_metadata_model = None
        self.sample_summary_model = None
        self.masses_model = None

    @pyqtSlot()
    def on_action_csvFiles(self):
        """Invoked when pressing Export CSV files."""
        if self.ls_data is None:
            QMessageBox.warning(self, 'Warning!', 'No data available!')
            return

        if (self.ls_data.df_panel_data is None) and (self.ls_data.df_panel_data is None) and (self.ls_data.df_masses is None):
            QMessageBox.warning(self, 'Warning!', 'No LifeScale data available!')
            return
        self.export_csv()

    def export_csv(self):
        """Export ls data to csv files."""

        # Get output directory:
        output_dir_name = QFileDialog.getExistingDirectory(self,
                                                           caption='Select an output directory',
                                                           directory=self.get_work_dir(),
                                                           options=QFileDialog.Option.ShowDirsOnly)
        # Write files
        try:
            self.ls_data.export_csv_files(output_dir_name, sort_by_time=False)
        except Exception as e:
            QMessageBox.critical(self, 'Error!', str(e))
            self.statusBar().showMessage(f'No LS data exportet.')
        else:
            self.status.showMessage(f'Exported CSV files to: {output_dir_name}')

    @pyqtSlot()
    def on_checkBox_plotMedian_stateChanged(self):
        """Invoked whenever the state changes."""
        if self.sample_histogram is not None:
            if self.masses_model is not None:
                self.plot_hist_mean_median_marker(self.masses_model.get_current_sample_id)

    @pyqtSlot()
    def on_checkBox_plotMean_stateChanged(self):
        """Invoked whenever the state changes."""
        if self.sample_histogram is not None:
            if self.masses_model is not None:
                self.plot_hist_mean_median_marker(self.masses_model.get_current_sample_id)

    @pyqtSlot()
    def on_pushButton_resetMassLimits_pressed(self):
        """Invoked whenever the button `pushButton_resetMassLimits` is pressed."""
        self.ls_data.reset_flag_masses(self.masses_model.get_current_sample_id)
        self.ls_data.calc_sample_statistics(selection_only=True)
        self.plot_masses_histogram(self.masses_model.get_current_sample_id)
        self.print_sample_statistics(self.masses_model.get_current_sample_id)

    def set_up_sample_histogram(self):
        """Set up `self.GraphicsLayoutWidget_stations_map` widget."""
        self.glw_sample_histogram = self.GraphicsLayoutWidget_SampelHistogram
        self.glw_sample_histogram.setBackground('w')  # white background color
        # Create sub-plots:
        self.sample_histogram = self.glw_sample_histogram.addPlot(0, 0, name='sample_histogram')
        self.sample_histogram.setLabel(axis='left', text='Number of Samples')
        self.sample_histogram.setLabel(axis='bottom', text='Mass')
        self.sample_histogram.setTitle('Sample Histogram')
        # self.stations_map.addLegend()
        self.sample_histogram.setMouseEnabled(y=False)

    def set_up_hist_method_comboBox(self):
        """Set up the histogram method combo box."""
        self.comboBox_histMethod.addItems(list(NUMPY_HISTOGRAM_BIN_EDGES_OPTIONS.keys()))
        # self.histogram_bin_method_selection(0)  # default: i. item in dict

    @pyqtSlot()
    def on_histogram_bin_method_currentIndexChanged(self):
        """Select bin determination method."""
        self.settings['hist_bin_mode'] = self.comboBox_histMethod.currentText()
        self.settings['hist_number_of_bins'] = self.spinBox_histBins.value()
        self.label_histBinMethodDescription.setText(NUMPY_HISTOGRAM_BIN_EDGES_OPTIONS[self.settings['hist_bin_mode']])
        if self.settings['hist_bin_mode'] == 'Num. of bins':
            self.label_numberOfBins.setEnabled(True)
            self.spinBox_histBins.setEnabled(True)
        else:
            self.label_numberOfBins.setEnabled(False)
            self.spinBox_histBins.setEnabled(False)

    def get_hist_bin_method(self):
        """Get selected bin method from GUI."""
        bins = self.comboBox_histMethod.currentText()
        if bins == 'Num. of bins':
            bins = self.spinBox_histBins.value()
        return bins

    @pyqtSlot(int)
    def on_checkBox_MassesShowTable_stateChanged(self, state):
        """Invoked whenever the state of the shoe massed checkbox changes."""
        self.set_model_tableView_Masses_based_on_checkbox_state()

    def set_model_tableView_Masses_based_on_checkbox_state(self):
        """Connect/disconnect the model of the masses table view based on checkbox state."""
        if self.checkBox_MassesShowTable.isChecked():
            self.tableView_Masses.setModel(self.masses_model)
        else:
            self.tableView_Masses.setModel(None)

    @pyqtSlot()
    def on_menu_file_load_xlsm_file(self):
        """Launch file selection dialog to load data from an xlsm file."""

        if self.ls_data is not None:
            reply = QMessageBox.question(self,
                                         'Message',
                                         f'Overwrite data from: {self.ls_data.run_name}',
                                         QMessageBox.StandardButton.Yes | QMessageBox.StandardButton.No,
                                         QMessageBox.StandardButton.No)
            if reply == QMessageBox.StandardButton.No:
                return
            self.reset_gui()  # Clear and reset all items

        xlsm_filename, _ = QFileDialog.getOpenFileName(self,
                                                       caption='Select LifeScale data file (xlsm format)',
                                                       directory=self.get_work_dir(),
                                                       filter="LifeScale data file (*.xlsm)")

        if xlsm_filename:
            xlsm_filename = QDir.toNativeSeparators(xlsm_filename)
            try:
                ls_data = LSData.from_xlsm_file(xlsm_filename)
                ls_data.calc_sample_statistics()
            except Exception as e:
                QMessageBox.critical(self, 'Error!', str(e))
                self.statusBar().showMessage(f"No LS data loaded.")
            else:
                self.ls_data = ls_data
                self.status.showMessage(f'Loaded: {ls_data}')
                self.setWindowTitle(self.ls_data.run_name)
                self.set_up_panel_metadata_view_model()
                self.set_up_sample_summary_view_model()
                self.set_up_masses_view_model()
                self.update_sample_list()
        else:
            self.statusBar().showMessage(f"No LS data loaded.")

    def set_up_sample_list(self):
        """Set up sample list widget"""
        self.sample_list = self.listWidget_Samples

    def print_sample_statistics(self, sample_id):
        """Print sample statistics in GUI."""
        sample_data = self.ls_data.df_interval_analysis.loc[self.ls_data.df_interval_analysis['SampleID'] == sample_id]
        if len(sample_data) == 0:
            raise IndexError(f'No sample dataset found for sample ID = {sample_id}')

        self.label_median.setText(
            f'{sample_data["Mass_median"].item():.3f} ({sample_data["Mass_median_keep"].item():.3f})')
        self.label_mean.setText(f'{sample_data["MeanMass"].item():.3f} ({sample_data["Mass_mean_keep"].item():.3f})')
        self.label_std.setText(f'{sample_data["Mass_std"].item():.3f} ({sample_data["Mass_std_keep"].item():.3f})')
        self.label_iqr.setText(f'{sample_data["Mass_iqr"].item():.3f} ({sample_data["Mass_iqr_keep"].item():.3f})')
        self.label_rcv.setText(
            f'{sample_data["Mass_rCV"].item():.3f} ({sample_data["Mass_rCV_keep"].item():.3f})')
        if sample_data["keep_min_mass"].isna().item() and sample_data["keep_max_mass"].isna().item():
            self.label_selectionLimits.clear()
            self.label_SelectedMasse.setText('No selection')
        else:
            self.label_selectionLimits.setText(
                f'[{sample_data["keep_min_mass"].item():.3f}, {sample_data["keep_max_mass"].item():.3f}]')
            num_selected = sample_data['DetectedParticles_keep'].item()
            num_all = sample_data['DetectedParticles'].item()
            selected_percent = (num_selected / num_all) * 100
            self.label_SelectedMasse.setText(f'{num_selected:.0f} of {num_all:.0f} ({selected_percent:.3f} %)')

    def update_sample_list(self):
        """Update sample list widget and add """
        # TODO: SampleDescription zu jedem Sample hinzufügen, siehe: ls_data.df_panel_data.SampleDescription
        self.sample_list.clear()
        # self.sample_list.addItems(self.ls_data.get_sample_ids)
        self.sample_list.addItems(self.sample_summary_model.get_sample_list_tags)

    @pyqtSlot()
    def on_selection_changed_sample_list(self):
        """Invoked whenever the item selection changed."""
        items = self.sample_list.selectedItems()
        if len(items) == 1:
            sample_id = items[0].text().split(':')[0]  # Extract sample ID from tag
            # TODO: Update gui models, plots, tables and statistics!
            self.update_masses_table_view(sample_id)
            self.status.showMessage(f'{len(self.masses_model.get_data)} mass(es) in sample {sample_id}.')
            self.plot_masses_histogram(sample_id)
            self.print_sample_statistics(sample_id)

        else:
            print('No or multiple items selected!')

    def plot_masses_histogram(self, sample_id, bins='auto'):
        """Create a histogram of masses of a single sample."""
        # ## compute standard histogram
        # y, x = np.histogram(vals, bins=np.linspace(-3, 8, 40))
        #
        # ## notice that len(x) == len(y)+1
        # ## We are required to use stepMode=True so that PlotCurveItem will interpret this data correctly.
        # curve = pg.PlotCurveItem(x, y, stepMode=True, fillLevel=0, brush=(0, 0, 255, 80))
        # plt1.addItem(curve)
        #
        # Get min and max mass threshold:
        tmp_filter = self.ls_data.df_interval_analysis['SampleID'] == sample_id
        keep_min_mass = self.ls_data.df_interval_analysis.loc[tmp_filter, 'keep_min_mass'].item()
        keep_max_mass = self.ls_data.df_interval_analysis.loc[tmp_filter, 'keep_max_mass'].item()

        # Get bin method:
        bins = self.get_hist_bin_method()

        self.sample_histogram.clear()
        masses = self.masses_model.get_data['Mass']
        y, x = np.histogram(masses, bins=bins)
        self.hist = pg.PlotCurveItem(x, y, stepMode=True, fillLevel=0, brush=(0, 0, 255, 80))
        self.sample_histogram.addItem(self.hist)

        # Get region limits:
        if keep_min_mass is None:
            x_min = x.min()
        else:
            x_min = keep_min_mass
        if keep_max_mass is None:
            x_max = x.max()
        else:
            x_max = keep_max_mass

        # Linear Reagions:
        pen = pg.mkPen(color='r', width=10)
        # https://pyqtgraph.readthedocs.io/en/latest/api_reference/graphicsItems/linearregionitem.html
        self.masses_region = pg.LinearRegionItem([x_min, x_max],
                                                 # bounds=[x.min(), x.max()],  # use clipItem instead
                                                 swapMode='block',
                                                 clipItem=self.hist,
                                                 hoverPen=pen)
        self.masses_region.setZValue(-10)
        self.sample_histogram.addItem(self.masses_region)
        self.masses_region.sigRegionChangeFinished.connect(self.on_masses_region_sigRegionChangeFinished)

        # Mean and median marker:
        self.plot_hist_mean_median_marker(sample_id)

    def plot_hist_mean_median_marker(self, sample_id: str):
        """Plot mean and median marker to the histogram plot."""
        if self.sample_histogram.legend is not None:
            self.sample_histogram.legend.clear()
        if self.hist_marker_mean_all is not None:
            self.hist_marker_mean_all.clear()
        if self.hist_marker_mean_selected is not None:
            self.hist_marker_mean_selected.clear()
        if self.hist_marker_median_all is not None:
            self.hist_marker_median_all.clear()
        if self.hist_marker_median_selected is not None:
            self.hist_marker_median_selected.clear()

        if self.checkBox_plotMean.isChecked() or self.checkBox_plotMedian.isChecked():

            self.sample_histogram.addLegend()

            tmp_filter = self.ls_data.df_interval_analysis['SampleID'] == sample_id
            mean_all = self.ls_data.df_interval_analysis.loc[tmp_filter, 'MeanMass'].item()
            mean_keep = self.ls_data.df_interval_analysis.loc[tmp_filter, 'Mass_mean_keep'].item()
            median_all = self.ls_data.df_interval_analysis.loc[tmp_filter, 'Mass_median'].item()
            median_keep = self.ls_data.df_interval_analysis.loc[tmp_filter, 'Mass_median_keep'].item()

            # y_max = self.sample_histogram.getAxis('left').range[1]
            y_max = self.hist.getData()[1].max()

            if self.checkBox_plotMean.isChecked():
                pen = pg.mkPen(color='b', width=1)
                self.hist_marker_mean_all = self.sample_histogram.plot([mean_all, mean_all], [0.0, y_max],
                                                                       name=f'Mean (all)',
                                                                       pen=pen)
                pen = pg.mkPen(color='r', width=1)
                self.hist_marker_mean_selected = self.sample_histogram.plot([mean_keep, mean_keep], [0.0, y_max],
                                                                            name=f'Mean (selected)',
                                                                            pen=pen)
            if self.checkBox_plotMedian.isChecked():
                pen = pg.mkPen(color='b', width=1, style=Qt.PenStyle.DotLine)
                self.hist_marker_median_all = self.sample_histogram.plot([median_all, median_all], [0.0, y_max],
                                                                         name=f'Median (all)',
                                                                         pen=pen)
                pen = pg.mkPen(color='r', width=1, style=Qt.PenStyle.DotLine)
                self.hist_marker_median_selected = self.sample_histogram.plot([median_keep, median_keep], [0.0, y_max],
                                                                              name=f'Median (selected)',
                                                                              pen=pen)

    @pyqtSlot()
    def on_masses_region_sigRegionChangeFinished(self):
        """Invoked whenever the masses selection region changed."""
        [min_mass, max_mass] = self.masses_region.getRegion()
        print(f'Mass selection changed: min={min_mass:.3f}, max={max_mass:.3f}')
        info_string = self.ls_data.flag_masses(min_mass,
                                               max_mass,
                                               sample_id=self.masses_model.get_current_sample_id)  # Set flag in ls_data object
        self.status.showMessage(info_string)
        self.update_masses_table_view(sample_id=self.masses_model.get_current_sample_id)
        self.ls_data.calc_sample_statistics(selection_only=True)
        self.print_sample_statistics(self.masses_model.get_current_sample_id)
        self.plot_hist_mean_median_marker(self.masses_model.get_current_sample_id)

    def reset_gui(self):
        """Clean and reset all GUI items, e.g. when loading new data."""
        self.sample_list.clear()

    def set_up_panel_metadata_view_model(self):
        """Set up the panel metadata table view model."""
        try:
            self.panel_metadata_model = PanelMetadataModel(self.ls_data.df_panel_data)
        except AttributeError:
            QMessageBox.warning(self, 'Warning!', 'No panel metadata available!')
        except Exception as e:
            QMessageBox.critical(self, 'Error!', str(e))
        else:
            self.tableView_PanelMetadata.setModel(self.panel_metadata_model)
            self.tableView_PanelMetadata.resizeColumnsToContents()

    def set_up_sample_summary_view_model(self):
        """Set up the sample summary table view model."""
        try:
            self.sample_summary_model = SampleSummaryModel(self.ls_data.df_interval_analysis)
        except AttributeError:
            QMessageBox.warning(self, 'Warning!', 'No sample summary data available!')
        except Exception as e:
            QMessageBox.critical(self, 'Error!', str(e))
        else:
            self.tableView_SampleSummary.setModel(self.sample_summary_model)
            self.tableView_SampleSummary.resizeColumnsToContents()

    def update_sample_summary_view_model(self):
        """Update the sample summary table view after changing the model."""
        pass

    def set_up_masses_view_model(self):
        """Set up the masses table view model."""
        try:
            self.masses_model = MassesModel(self.ls_data.df_masses)
        except AttributeError:
            QMessageBox.warning(self, 'Warning!', 'No masses data available!')
        except Exception as e:
            QMessageBox.critical(self, 'Error!', str(e))
        else:
            self.set_model_tableView_Masses_based_on_checkbox_state()
            self.tableView_Masses.resizeColumnsToContents()
            # self.masses_model.dataChanged.connect(self.on_masses_model_data_changed)

    def update_masses_table_view(self, sample_id: int):
        """Update the masses table view according to the sample selection"""
        self.masses_model.update_view_model(sample_id)
        self.masses_model.layoutChanged.emit()  # Show changes in table view
        # Workaround: https://stackoverflow.com/questions/8199747/sizing-to-fit-rows-and-columns-in-a-qtableview-is-very-slow
        # self.tableView_Masses.resizeColumnsToContents()  # Takes a very long time to execute!

    def closeEvent(self, event):
        """Ask the user whether to close the main window or not!"""
        reply = QMessageBox.question(self,
                                     'Message',
                                     "Are you sure to quit?",
                                     QMessageBox.StandardButton.Yes | QMessageBox.StandardButton.No,
                                     QMessageBox.StandardButton.No)
        if reply == QMessageBox.StandardButton.Yes:
            event.accept()
        else:
            event.ignore()

    @pyqtSlot()
    def exit_application(self):
        """Exit the application."""
        reply = QMessageBox.question(self,
                                     'Message',
                                     "Are you sure to quit?",
                                     QMessageBox.StandardButton.Yes | QMessageBox.StandardButton.No,
                                     QMessageBox.StandardButton.No)
        if reply == QMessageBox.StandardButton.Yes:
            sys.exit()

    def get_work_dir(self):
        """Returns the current working directory for file dialogs"""
        return os.getcwd()

    @pyqtSlot()
    def on_menu_help_about(self):
        """Launch the about dialog."""
        return_value = self.dlg_about.exec()


class DialogAbout(QDialog, Ui_Dialog_about):
    """About dialog."""

    def __init__(self, parent=None):
        super().__init__(parent)

        # Run the .setupUi() method to show the GUI
        self.setupUi(self)


def main():
    """Main program to start the GUI."""
    # Create the application
    app = QApplication(sys.argv)

    # Create and show the application's main window
    main_window = MainWindow()
    main_window.show()
    # Run the application's main loop:
    sys.exit(
        app.exec())  # exit or error code of Qt (app.exec_) is passed to sys.exit. Terminates pgm with standard
    # python method


if __name__ == "__main__":
    """Main Program."""
    main()



# Form implementation generated from reading ui file 'lifescale/gui/MainWindow.ui'
#
# Created by: PyQt6 UI code generator 6.4.0
#
# WARNING: Any manual changes made to this file will be lost when pyuic6 is
# run again.  Do not edit this file unless you know what you are doing.


from PyQt6 import QtCore, QtGui, QtWidgets


class Ui_MainWindow(object):
    def setupUi(self, MainWindow):
        MainWindow.setObjectName("MainWindow")
        MainWindow.resize(1277, 728)
        self.centralwidget = QtWidgets.QWidget(MainWindow)
        self.centralwidget.setObjectName("centralwidget")
        self.verticalLayout = QtWidgets.QVBoxLayout(self.centralwidget)
        self.verticalLayout.setObjectName("verticalLayout")
        self.tabWidget_Main = QtWidgets.QTabWidget(self.centralwidget)
        self.tabWidget_Main.setTabPosition(QtWidgets.QTabWidget.TabPosition.East)
        self.tabWidget_Main.setObjectName("tabWidget_Main")
        self.tab_LSData = QtWidgets.QWidget()
        self.tab_LSData.setObjectName("tab_LSData")
        self.verticalLayout_4 = QtWidgets.QVBoxLayout(self.tab_LSData)
        self.verticalLayout_4.setObjectName("verticalLayout_4")
        self.splitter = QtWidgets.QSplitter(self.tab_LSData)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Policy.Expanding, QtWidgets.QSizePolicy.Policy.Preferred)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.splitter.sizePolicy().hasHeightForWidth())
        self.splitter.setSizePolicy(sizePolicy)
        self.splitter.setOrientation(QtCore.Qt.Orientation.Horizontal)
        self.splitter.setChildrenCollapsible(False)
        self.splitter.setObjectName("splitter")
        self.layoutWidget = QtWidgets.QWidget(self.splitter)
        self.layoutWidget.setObjectName("layoutWidget")
        self.verticalLayout_2 = QtWidgets.QVBoxLayout(self.layoutWidget)
        self.verticalLayout_2.setSizeConstraint(QtWidgets.QLayout.SizeConstraint.SetDefaultConstraint)
        self.verticalLayout_2.setContentsMargins(0, 0, 0, 0)
        self.verticalLayout_2.setObjectName("verticalLayout_2")
        self.listWidget_Samples = QtWidgets.QListWidget(self.layoutWidget)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Policy.MinimumExpanding, QtWidgets.QSizePolicy.Policy.Expanding)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.listWidget_Samples.sizePolicy().hasHeightForWidth())
        self.listWidget_Samples.setSizePolicy(sizePolicy)
        self.listWidget_Samples.setMinimumSize(QtCore.QSize(150, 0))
        self.listWidget_Samples.setMaximumSize(QtCore.QSize(16777215, 16777215))
        self.listWidget_Samples.setObjectName("listWidget_Samples")
        self.verticalLayout_2.addWidget(self.listWidget_Samples)
        self.groupBox_LSDataOptions = QtWidgets.QGroupBox(self.layoutWidget)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Policy.MinimumExpanding, QtWidgets.QSizePolicy.Policy.Preferred)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.groupBox_LSDataOptions.sizePolicy().hasHeightForWidth())
        self.groupBox_LSDataOptions.setSizePolicy(sizePolicy)
        self.groupBox_LSDataOptions.setMinimumSize(QtCore.QSize(150, 0))
        self.groupBox_LSDataOptions.setMaximumSize(QtCore.QSize(16777215, 16777215))
        self.groupBox_LSDataOptions.setObjectName("groupBox_LSDataOptions")
        self.verticalLayout_8 = QtWidgets.QVBoxLayout(self.groupBox_LSDataOptions)
        self.verticalLayout_8.setObjectName("verticalLayout_8")
        self.pushButton_resetMassLimits = QtWidgets.QPushButton(self.groupBox_LSDataOptions)
        self.pushButton_resetMassLimits.setObjectName("pushButton_resetMassLimits")
        self.verticalLayout_8.addWidget(self.pushButton_resetMassLimits)
        self.checkBox_MassesShowTable = QtWidgets.QCheckBox(self.groupBox_LSDataOptions)
        self.checkBox_MassesShowTable.setChecked(True)
        self.checkBox_MassesShowTable.setObjectName("checkBox_MassesShowTable")
        self.verticalLayout_8.addWidget(self.checkBox_MassesShowTable)
        self.verticalLayout_2.addWidget(self.groupBox_LSDataOptions)
        self.tabWidget_LSData = QtWidgets.QTabWidget(self.splitter)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Policy.Expanding, QtWidgets.QSizePolicy.Policy.Expanding)
        sizePolicy.setHorizontalStretch(1)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.tabWidget_LSData.sizePolicy().hasHeightForWidth())
        self.tabWidget_LSData.setSizePolicy(sizePolicy)
        self.tabWidget_LSData.setObjectName("tabWidget_LSData")
        self.tab_LSData_hist = QtWidgets.QWidget()
        self.tab_LSData_hist.setObjectName("tab_LSData_hist")
        self.verticalLayout_9 = QtWidgets.QVBoxLayout(self.tab_LSData_hist)
        self.verticalLayout_9.setObjectName("verticalLayout_9")
        self.GraphicsLayoutWidget_SampelHistogram = GraphicsLayoutWidget(self.tab_LSData_hist)
        self.GraphicsLayoutWidget_SampelHistogram.setObjectName("GraphicsLayoutWidget_SampelHistogram")
        self.verticalLayout_9.addWidget(self.GraphicsLayoutWidget_SampelHistogram)
        self.horizontalLayout_2 = QtWidgets.QHBoxLayout()
        self.horizontalLayout_2.setObjectName("horizontalLayout_2")
        self.groupBoxLSDataSampleStatistics = QtWidgets.QGroupBox(self.tab_LSData_hist)
        self.groupBoxLSDataSampleStatistics.setObjectName("groupBoxLSDataSampleStatistics")
        self.horizontalLayout = QtWidgets.QHBoxLayout(self.groupBoxLSDataSampleStatistics)
        self.horizontalLayout.setObjectName("horizontalLayout")
        self.formLayout = QtWidgets.QFormLayout()
        self.formLayout.setObjectName("formLayout")
        self.label = QtWidgets.QLabel(self.groupBoxLSDataSampleStatistics)
        self.label.setObjectName("label")
        self.formLayout.setWidget(0, QtWidgets.QFormLayout.ItemRole.LabelRole, self.label)
        self.label_SelectedMasse = QtWidgets.QLabel(self.groupBoxLSDataSampleStatistics)
        self.label_SelectedMasse.setText("")
        self.label_SelectedMasse.setObjectName("label_SelectedMasse")
        self.formLayout.setWidget(0, QtWidgets.QFormLayout.ItemRole.FieldRole, self.label_SelectedMasse)
        self.label_3 = QtWidgets.QLabel(self.groupBoxLSDataSampleStatistics)
        self.label_3.setObjectName("label_3")
        self.formLayout.setWidget(1, QtWidgets.QFormLayout.ItemRole.LabelRole, self.label_3)
        self.label_selectionLimits = QtWidgets.QLabel(self.groupBoxLSDataSampleStatistics)
        self.label_selectionLimits.setText("")
        self.label_selectionLimits.setObjectName("label_selectionLimits")
        self.formLayout.setWidget(1, QtWidgets.QFormLayout.ItemRole.FieldRole, self.label_selectionLimits)
        self.label_5 = QtWidgets.QLabel(self.groupBoxLSDataSampleStatistics)
        self.label_5.setObjectName("label_5")
        self.formLayout.setWidget(2, QtWidgets.QFormLayout.ItemRole.LabelRole, self.label_5)
        self.label_mean = QtWidgets.QLabel(self.groupBoxLSDataSampleStatistics)
        self.label_mean.setText("")
        self.label_mean.setObjectName("label_mean")
        self.formLayout.setWidget(2, QtWidgets.QFormLayout.ItemRole.FieldRole, self.label_mean)
        self.label_7 = QtWidgets.QLabel(self.groupBoxLSDataSampleStatistics)
        self.label_7.setObjectName("label_7")
        self.formLayout.setWidget(3, QtWidgets.QFormLayout.ItemRole.LabelRole, self.label_7)
        self.label_median = QtWidgets.QLabel(self.groupBoxLSDataSampleStatistics)
        self.label_median.setText("")
        self.label_median.setObjectName("label_median")
        self.formLayout.setWidget(3, QtWidgets.QFormLayout.ItemRole.FieldRole, self.label_median)
        self.label_9 = QtWidgets.QLabel(self.groupBoxLSDataSampleStatistics)
        self.label_9.setObjectName("label_9")
        self.formLayout.setWidget(4, QtWidgets.QFormLayout.ItemRole.LabelRole, self.label_9)
        self.label_std = QtWidgets.QLabel(self.groupBoxLSDataSampleStatistics)
        self.label_std.setText("")
        self.label_std.setObjectName("label_std")
        self.formLayout.setWidget(4, QtWidgets.QFormLayout.ItemRole.FieldRole, self.label_std)
        self.label_11 = QtWidgets.QLabel(self.groupBoxLSDataSampleStatistics)
        self.label_11.setObjectName("label_11")
        self.formLayout.setWidget(5, QtWidgets.QFormLayout.ItemRole.LabelRole, self.label_11)
        self.label_iqr = QtWidgets.QLabel(self.groupBoxLSDataSampleStatistics)
        self.label_iqr.setText("")
        self.label_iqr.setObjectName("label_iqr")
        self.formLayout.setWidget(5, QtWidgets.QFormLayout.ItemRole.FieldRole, self.label_iqr)
        self.label_13 = QtWidgets.QLabel(self.groupBoxLSDataSampleStatistics)
        self.label_13.setObjectName("label_13")
        self.formLayout.setWidget(6, QtWidgets.QFormLayout.ItemRole.LabelRole, self.label_13)
        self.label_rcv = QtWidgets.QLabel(self.groupBoxLSDataSampleStatistics)
        self.label_rcv.setText("")
        self.label_rcv.setObjectName("label_rcv")
        self.formLayout.setWidget(6, QtWidgets.QFormLayout.ItemRole.FieldRole, self.label_rcv)
        self.horizontalLayout.addLayout(self.formLayout)
        self.horizontalLayout_2.addWidget(self.groupBoxLSDataSampleStatistics)
        self.groupBox_histOptions = QtWidgets.QGroupBox(self.tab_LSData_hist)
        self.groupBox_histOptions.setObjectName("groupBox_histOptions")
        self.verticalLayout_3 = QtWidgets.QVBoxLayout(self.groupBox_histOptions)
        self.verticalLayout_3.setObjectName("verticalLayout_3")
        self.formLayout_2 = QtWidgets.QFormLayout()
        self.formLayout_2.setObjectName("formLayout_2")
        self.label_2 = QtWidgets.QLabel(self.groupBox_histOptions)
        self.label_2.setObjectName("label_2")
        self.formLayout_2.setWidget(0, QtWidgets.QFormLayout.ItemRole.LabelRole, self.label_2)
        self.comboBox_histMethod = QtWidgets.QComboBox(self.groupBox_histOptions)
        self.comboBox_histMethod.setObjectName("comboBox_histMethod")
        self.formLayout_2.setWidget(0, QtWidgets.QFormLayout.ItemRole.FieldRole, self.comboBox_histMethod)
        self.label_numberOfBins = QtWidgets.QLabel(self.groupBox_histOptions)
        self.label_numberOfBins.setObjectName("label_numberOfBins")
        self.formLayout_2.setWidget(1, QtWidgets.QFormLayout.ItemRole.LabelRole, self.label_numberOfBins)
        self.spinBox_histBins = QtWidgets.QSpinBox(self.groupBox_histOptions)
        self.spinBox_histBins.setMaximum(1000)
        self.spinBox_histBins.setStepType(QtWidgets.QAbstractSpinBox.StepType.AdaptiveDecimalStepType)
        self.spinBox_histBins.setProperty("value", 100)
        self.spinBox_histBins.setObjectName("spinBox_histBins")
        self.formLayout_2.setWidget(1, QtWidgets.QFormLayout.ItemRole.FieldRole, self.spinBox_histBins)
        self.verticalLayout_3.addLayout(self.formLayout_2)
        self.label_histBinMethodDescription = QtWidgets.QLabel(self.groupBox_histOptions)
        self.label_histBinMethodDescription.setText("")
        self.label_histBinMethodDescription.setWordWrap(True)
        self.label_histBinMethodDescription.setObjectName("label_histBinMethodDescription")
        self.verticalLayout_3.addWidget(self.label_histBinMethodDescription)
        self.checkBox_plotMedian = QtWidgets.QCheckBox(self.groupBox_histOptions)
        self.checkBox_plotMedian.setChecked(False)
        self.checkBox_plotMedian.setObjectName("checkBox_plotMedian")
        self.verticalLayout_3.addWidget(self.checkBox_plotMedian)
        self.checkBox_plotMean = QtWidgets.QCheckBox(self.groupBox_histOptions)
        self.checkBox_plotMean.setChecked(False)
        self.checkBox_plotMean.setObjectName("checkBox_plotMean")
        self.verticalLayout_3.addWidget(self.checkBox_plotMean)
        self.horizontalLayout_2.addWidget(self.groupBox_histOptions)
        self.verticalLayout_9.addLayout(self.horizontalLayout_2)
        self.tabWidget_LSData.addTab(self.tab_LSData_hist, "")
        self.tab_Masses = QtWidgets.QWidget()
        self.tab_Masses.setObjectName("tab_Masses")
        self.verticalLayout_5 = QtWidgets.QVBoxLayout(self.tab_Masses)
        self.verticalLayout_5.setObjectName("verticalLayout_5")
        self.tableView_Masses = QtWidgets.QTableView(self.tab_Masses)
        self.tableView_Masses.setEnabled(True)
        self.tableView_Masses.setLineWidth(9)
        self.tableView_Masses.setSizeAdjustPolicy(QtWidgets.QAbstractScrollArea.SizeAdjustPolicy.AdjustToContents)
        self.tableView_Masses.setObjectName("tableView_Masses")
        self.verticalLayout_5.addWidget(self.tableView_Masses)
        self.tabWidget_LSData.addTab(self.tab_Masses, "")
        self.verticalLayout_4.addWidget(self.splitter)
        self.tabWidget_Main.addTab(self.tab_LSData, "")
        self.tab_SampleSummary = QtWidgets.QWidget()
        self.tab_SampleSummary.setObjectName("tab_SampleSummary")
        self.verticalLayout_7 = QtWidgets.QVBoxLayout(self.tab_SampleSummary)
        self.verticalLayout_7.setObjectName("verticalLayout_7")
        self.tableView_SampleSummary = QtWidgets.QTableView(self.tab_SampleSummary)
        self.tableView_SampleSummary.setObjectName("tableView_SampleSummary")
        self.verticalLayout_7.addWidget(self.tableView_SampleSummary)
        self.tabWidget_Main.addTab(self.tab_SampleSummary, "")
        self.tab_PanelMetadata = QtWidgets.QWidget()
        self.tab_PanelMetadata.setObjectName("tab_PanelMetadata")
        self.verticalLayout_6 = QtWidgets.QVBoxLayout(self.tab_PanelMetadata)
        self.verticalLayout_6.setObjectName("verticalLayout_6")
        self.tableView_PanelMetadata = QtWidgets.QTableView(self.tab_PanelMetadata)
        self.tableView_PanelMetadata.setObjectName("tableView_PanelMetadata")
        self.verticalLayout_6.addWidget(self.tableView_PanelMetadata)
        self.tabWidget_Main.addTab(self.tab_PanelMetadata, "")
        self.verticalLayout.addWidget(self.tabWidget_Main)
        MainWindow.setCentralWidget(self.centralwidget)
        self.menubar = QtWidgets.QMenuBar(MainWindow)
        self.menubar.setGeometry(QtCore.QRect(0, 0, 1277, 22))
        self.menubar.setObjectName("menubar")
        self.menuFile = QtWidgets.QMenu(self.menubar)
        self.menuFile.setObjectName("menuFile")
        self.menuExport = QtWidgets.QMenu(self.menuFile)
        self.menuExport.setObjectName("menuExport")
        self.menuHelp = QtWidgets.QMenu(self.menubar)
        self.menuHelp.setObjectName("menuHelp")
        MainWindow.setMenuBar(self.menubar)
        self.statusbar = QtWidgets.QStatusBar(MainWindow)
        self.statusbar.setObjectName("statusbar")
        MainWindow.setStatusBar(self.statusbar)
        self.actionLoad_xlsm_file = QtGui.QAction(MainWindow)
        self.actionLoad_xlsm_file.setObjectName("actionLoad_xlsm_file")
        self.action_About = QtGui.QAction(MainWindow)
        self.action_About.setObjectName("action_About")
        self.action_csvFiles = QtGui.QAction(MainWindow)
        self.action_csvFiles.setObjectName("action_csvFiles")
        self.action_Exit = QtGui.QAction(MainWindow)
        self.action_Exit.setObjectName("action_Exit")
        self.menuExport.addAction(self.action_csvFiles)
        self.menuFile.addAction(self.actionLoad_xlsm_file)
        self.menuFile.addAction(self.menuExport.menuAction())
        self.menuFile.addSeparator()
        self.menuFile.addAction(self.action_Exit)
        self.menuHelp.addAction(self.action_About)
        self.menubar.addAction(self.menuFile.menuAction())
        self.menubar.addAction(self.menuHelp.menuAction())

        self.retranslateUi(MainWindow)
        self.tabWidget_Main.setCurrentIndex(0)
        self.tabWidget_LSData.setCurrentIndex(0)
        QtCore.QMetaObject.connectSlotsByName(MainWindow)

    def retranslateUi(self, MainWindow):
        _translate = QtCore.QCoreApplication.translate
        MainWindow.setWindowTitle(_translate("MainWindow", "LifeScale utils"))
        self.groupBox_LSDataOptions.setTitle(_translate("MainWindow", "Options"))
        self.pushButton_resetMassLimits.setText(_translate("MainWindow", "Reset mass limits"))
        self.checkBox_MassesShowTable.setText(_translate("MainWindow", "Show table"))
        self.groupBoxLSDataSampleStatistics.setTitle(_translate("MainWindow", "Sample statistics (current selection)"))
        self.label.setText(_translate("MainWindow", "Selected masses"))
        self.label_3.setText(_translate("MainWindow", "Selection limits"))
        self.label_5.setText(_translate("MainWindow", "Mean"))
        self.label_7.setText(_translate("MainWindow", "Median"))
        self.label_9.setText(_translate("MainWindow", "Std"))
        self.label_11.setText(_translate("MainWindow", "IQR"))
        self.label_13.setText(_translate("MainWindow", "rCV"))
        self.groupBox_histOptions.setTitle(_translate("MainWindow", "Histogram options"))
        self.label_2.setText(_translate("MainWindow", "Bin determination method"))
        self.label_numberOfBins.setText(_translate("MainWindow", "Number of bins"))
        self.checkBox_plotMedian.setText(_translate("MainWindow", "Plot median"))
        self.checkBox_plotMean.setText(_translate("MainWindow", "Plot mean"))
        self.tabWidget_LSData.setTabText(self.tabWidget_LSData.indexOf(self.tab_LSData_hist), _translate("MainWindow", "Sample histogram and statistics"))
        self.tabWidget_LSData.setTabText(self.tabWidget_LSData.indexOf(self.tab_Masses), _translate("MainWindow", "Masses per Sample"))
        self.tabWidget_Main.setTabText(self.tabWidget_Main.indexOf(self.tab_LSData), _translate("MainWindow", "Masses"))
        self.tabWidget_Main.setTabText(self.tabWidget_Main.indexOf(self.tab_SampleSummary), _translate("MainWindow", "Sample Summary"))
        self.tabWidget_Main.setTabText(self.tabWidget_Main.indexOf(self.tab_PanelMetadata), _translate("MainWindow", "Panel Metadata"))
        self.menuFile.setTitle(_translate("MainWindow", "File"))
        self.menuExport.setTitle(_translate("MainWindow", "Export"))
        self.menuHelp.setTitle(_translate("MainWindow", "Help"))
        self.actionLoad_xlsm_file.setText(_translate("MainWindow", "Load xlsm file"))
        self.actionLoad_xlsm_file.setToolTip(_translate("MainWindow", "Load xlsm file created by LifeScale"))
        self.action_About.setText(_translate("MainWindow", "About"))
        self.action_csvFiles.setText(_translate("MainWindow", "CSV files"))
        self.action_Exit.setText(_translate("MainWindow", "Exit"))
from pyqtgraph import GraphicsLayoutWidget



# Form implementation generated from reading ui file 'lifescale/gui/dialog_about.ui'
#
# Created by: PyQt6 UI code generator 6.4.0
#
# WARNING: Any manual changes made to this file will be lost when pyuic6 is
# run again.  Do not edit this file unless you know what you are doing.


from PyQt6 import QtCore, QtGui, QtWidgets


class Ui_Dialog_about(object):
    def setupUi(self, Dialog_about):
        Dialog_about.setObjectName("Dialog_about")
        Dialog_about.resize(610, 677)
        Dialog_about.setMinimumSize(QtCore.QSize(610, 500))
        self.verticalLayout_3 = QtWidgets.QVBoxLayout(Dialog_about)
        self.verticalLayout_3.setObjectName("verticalLayout_3")
        self.label_2 = QtWidgets.QLabel(Dialog_about)
        font = QtGui.QFont()
        font.setPointSize(12)
        font.setBold(True)
        font.setWeight(75)
        self.label_2.setFont(font)
        self.label_2.setTextFormat(QtCore.Qt.TextFormat.AutoText)
        self.label_2.setWordWrap(True)
        self.label_2.setObjectName("label_2")
        self.verticalLayout_3.addWidget(self.label_2)
        self.formLayout = QtWidgets.QFormLayout()
        self.formLayout.setObjectName("formLayout")
        self.label_8 = QtWidgets.QLabel(Dialog_about)
        self.label_8.setObjectName("label_8")
        self.formLayout.setWidget(0, QtWidgets.QFormLayout.ItemRole.LabelRole, self.label_8)
        self.label_version = QtWidgets.QLabel(Dialog_about)
        self.label_version.setObjectName("label_version")
        self.formLayout.setWidget(0, QtWidgets.QFormLayout.ItemRole.FieldRole, self.label_version)
        self.label_6 = QtWidgets.QLabel(Dialog_about)
        self.label_6.setObjectName("label_6")
        self.formLayout.setWidget(1, QtWidgets.QFormLayout.ItemRole.LabelRole, self.label_6)
        self.label_email = QtWidgets.QLabel(Dialog_about)
        self.label_email.setObjectName("label_email")
        self.formLayout.setWidget(1, QtWidgets.QFormLayout.ItemRole.FieldRole, self.label_email)
        self.label_7 = QtWidgets.QLabel(Dialog_about)
        self.label_7.setObjectName("label_7")
        self.formLayout.setWidget(3, QtWidgets.QFormLayout.ItemRole.LabelRole, self.label_7)
        self.label_copyright = QtWidgets.QLabel(Dialog_about)
        self.label_copyright.setObjectName("label_copyright")
        self.formLayout.setWidget(3, QtWidgets.QFormLayout.ItemRole.FieldRole, self.label_copyright)
        self.label_9 = QtWidgets.QLabel(Dialog_about)
        self.label_9.setObjectName("label_9")
        self.formLayout.setWidget(4, QtWidgets.QFormLayout.ItemRole.LabelRole, self.label_9)
        self.label_git_repo = QtWidgets.QLabel(Dialog_about)
        self.label_git_repo.setOpenExternalLinks(True)
        self.label_git_repo.setObjectName("label_git_repo")
        self.formLayout.setWidget(4, QtWidgets.QFormLayout.ItemRole.FieldRole, self.label_git_repo)
        self.label_5 = QtWidgets.QLabel(Dialog_about)
        self.label_5.setObjectName("label_5")
        self.formLayout.setWidget(5, QtWidgets.QFormLayout.ItemRole.LabelRole, self.label_5)
        self.label_pypi_repo = QtWidgets.QLabel(Dialog_about)
        self.label_pypi_repo.setObjectName("label_pypi_repo")
        self.formLayout.setWidget(5, QtWidgets.QFormLayout.ItemRole.FieldRole, self.label_pypi_repo)
        self.label_10 = QtWidgets.QLabel(Dialog_about)
        self.label_10.setObjectName("label_10")
        self.formLayout.setWidget(2, QtWidgets.QFormLayout.ItemRole.LabelRole, self.label_10)
        self.label_author = QtWidgets.QLabel(Dialog_about)
        self.label_author.setObjectName("label_author")
        self.formLayout.setWidget(2, QtWidgets.QFormLayout.ItemRole.FieldRole, self.label_author)
        self.verticalLayout_3.addLayout(self.formLayout)
        spacerItem = QtWidgets.QSpacerItem(20, 10, QtWidgets.QSizePolicy.Policy.Minimum, QtWidgets.QSizePolicy.Policy.Fixed)
        self.verticalLayout_3.addItem(spacerItem)
        self.horizontalLayout = QtWidgets.QHBoxLayout()
        self.horizontalLayout.setObjectName("horizontalLayout")
        self.label_4 = QtWidgets.QLabel(Dialog_about)
        self.label_4.setWordWrap(True)
        self.label_4.setOpenExternalLinks(True)
        self.label_4.setObjectName("label_4")
        self.horizontalLayout.addWidget(self.label_4)
        self.label = QtWidgets.QLabel(Dialog_about)
        self.label.setText("")
        self.label.setPixmap(QtGui.QPixmap(":/logos/logos/Uni_Logo_2016_SW_230px-wide.jpg"))
        self.label.setScaledContents(False)
        self.label.setObjectName("label")
        self.horizontalLayout.addWidget(self.label)
        self.verticalLayout_3.addLayout(self.horizontalLayout)
        spacerItem1 = QtWidgets.QSpacerItem(20, 10, QtWidgets.QSizePolicy.Policy.Minimum, QtWidgets.QSizePolicy.Policy.Fixed)
        self.verticalLayout_3.addItem(spacerItem1)
        self.groupBox = QtWidgets.QGroupBox(Dialog_about)
        self.groupBox.setObjectName("groupBox")
        self.verticalLayout_2 = QtWidgets.QVBoxLayout(self.groupBox)
        self.verticalLayout_2.setObjectName("verticalLayout_2")
        self.verticalLayout = QtWidgets.QVBoxLayout()
        self.verticalLayout.setObjectName("verticalLayout")
        self.label_3 = QtWidgets.QLabel(self.groupBox)
        self.label_3.setOpenExternalLinks(True)
        self.label_3.setObjectName("label_3")
        self.verticalLayout.addWidget(self.label_3)
        self.plainTextEdit = QtWidgets.QPlainTextEdit(self.groupBox)
        font = QtGui.QFont()
        font.setPointSize(10)
        self.plainTextEdit.setFont(font)
        self.plainTextEdit.setObjectName("plainTextEdit")
        self.verticalLayout.addWidget(self.plainTextEdit)
        self.verticalLayout_2.addLayout(self.verticalLayout)
        self.verticalLayout_3.addWidget(self.groupBox)
        self.buttonBox = QtWidgets.QDialogButtonBox(Dialog_about)
        self.buttonBox.setOrientation(QtCore.Qt.Orientation.Horizontal)
        self.buttonBox.setStandardButtons(QtWidgets.QDialogButtonBox.StandardButton.Close)
        self.buttonBox.setObjectName("buttonBox")
        self.verticalLayout_3.addWidget(self.buttonBox)

        self.retranslateUi(Dialog_about)
        self.buttonBox.accepted.connect(Dialog_about.accept) # type: ignore
        self.buttonBox.rejected.connect(Dialog_about.reject) # type: ignore
        QtCore.QMetaObject.connectSlotsByName(Dialog_about)

    def retranslateUi(self, Dialog_about):
        _translate = QtCore.QCoreApplication.translate
        Dialog_about.setWindowTitle(_translate("Dialog_about", "About"))
        self.label_2.setText(_translate("Dialog_about", "LifeScale utils"))
        self.label_8.setText(_translate("Dialog_about", "Version:"))
        self.label_version.setText(_translate("Dialog_about", "version"))
        self.label_6.setText(_translate("Dialog_about", "Contact:"))
        self.label_email.setText(_translate("Dialog_about", "email"))
        self.label_7.setText(_translate("Dialog_about", "Copyright:"))
        self.label_copyright.setText(_translate("Dialog_about", "copyright"))
        self.label_9.setText(_translate("Dialog_about", "Git repository:"))
        self.label_git_repo.setText(_translate("Dialog_about", "git_repo"))
        self.label_5.setText(_translate("Dialog_about", "pypi.org repository:"))
        self.label_pypi_repo.setText(_translate("Dialog_about", "pypi_repo"))
        self.label_10.setText(_translate("Dialog_about", "Author:"))
        self.label_author.setText(_translate("Dialog_about", "author"))
        self.label_4.setText(_translate("Dialog_about", "<html><head/><body><p>LifeScale utils was funded by the <a href=\"https://cmess.univie.ac.at/\"><span style=\" text-decoration: underline; color:#0000ff;\">Centre for Microbiology and Environmental Systems Science (CMESS) of the University of Vienna</span></a>.</p></body></html>"))
        self.groupBox.setTitle(_translate("Dialog_about", "License"))
        self.label_3.setText(_translate("Dialog_about", "<html><head/><body><p>LifeScale utils is licensed under the <a href=\"https://www.gnu.org/licenses/gpl-3.0.de.html\"><span style=\" text-decoration: underline; color:#0000ff;\">GNU General Public License 3</span></a>.</p></body></html>"))
        self.plainTextEdit.setPlainText(_translate("Dialog_about", " GNU GENERAL PUBLIC LICENSE\n"
"                       Version 3, 29 June 2007\n"
"\n"
" Copyright (C) 2007 Free Software Foundation, Inc. <https://fsf.org/>\n"
" Everyone is permitted to copy and distribute verbatim copies\n"
" of this license document, but changing it is not allowed.\n"
"\n"
"                            Preamble\n"
"\n"
"  The GNU General Public License is a free, copyleft license for\n"
"software and other kinds of works.\n"
"\n"
"  The licenses for most software and other practical works are designed\n"
"to take away your freedom to share and change the works.  By contrast,\n"
"the GNU General Public License is intended to guarantee your freedom to\n"
"share and change all versions of a program--to make sure it remains free\n"
"software for all its users.  We, the Free Software Foundation, use the\n"
"GNU General Public License for most of our software; it applies also to\n"
"any other work released this way by its authors.  You can apply it to\n"
"your programs, too.\n"
"\n"
"  When we speak of free software, we are referring to freedom, not\n"
"price.  Our General Public Licenses are designed to make sure that you\n"
"have the freedom to distribute copies of free software (and charge for\n"
"them if you wish), that you receive source code or can get it if you\n"
"want it, that you can change the software or use pieces of it in new\n"
"free programs, and that you know you can do these things.\n"
"\n"
"  To protect your rights, we need to prevent others from denying you\n"
"these rights or asking you to surrender the rights.  Therefore, you have\n"
"certain responsibilities if you distribute copies of the software, or if\n"
"you modify it: responsibilities to respect the freedom of others.\n"
"\n"
"  For example, if you distribute copies of such a program, whether\n"
"gratis or for a fee, you must pass on to the recipients the same\n"
"freedoms that you received.  You must make sure that they, too, receive\n"
"or can get the source code.  And you must show them these terms so they\n"
"know their rights.\n"
"\n"
"  Developers that use the GNU GPL protect your rights with two steps:\n"
"(1) assert copyright on the software, and (2) offer you this License\n"
"giving you legal permission to copy, distribute and/or modify it.\n"
"\n"
"  For the developers\' and authors\' protection, the GPL clearly explains\n"
"that there is no warranty for this free software.  For both users\' and\n"
"authors\' sake, the GPL requires that modified versions be marked as\n"
"changed, so that their problems will not be attributed erroneously to\n"
"authors of previous versions.\n"
"\n"
"  Some devices are designed to deny users access to install or run\n"
"modified versions of the software inside them, although the manufacturer\n"
"can do so.  This is fundamentally incompatible with the aim of\n"
"protecting users\' freedom to change the software.  The systematic\n"
"pattern of such abuse occurs in the area of products for individuals to\n"
"use, which is precisely where it is most unacceptable.  Therefore, we\n"
"have designed this version of the GPL to prohibit the practice for those\n"
"products.  If such problems arise substantially in other domains, we\n"
"stand ready to extend this provision to those domains in future versions\n"
"of the GPL, as needed to protect the freedom of users.\n"
"\n"
"  Finally, every program is threatened constantly by software patents.\n"
"States should not allow patents to restrict development and use of\n"
"software on general-purpose computers, but in those that do, we wish to\n"
"avoid the special danger that patents applied to a free program could\n"
"make it effectively proprietary.  To prevent this, the GPL assures that\n"
"patents cannot be used to render the program non-free.\n"
"\n"
"  The precise terms and conditions for copying, distribution and\n"
"modification follow.\n"
"\n"
"                       TERMS AND CONDITIONS\n"
"\n"
"  0. Definitions.\n"
"\n"
"  \"This License\" refers to version 3 of the GNU General Public License.\n"
"\n"
"  \"Copyright\" also means copyright-like laws that apply to other kinds of\n"
"works, such as semiconductor masks.\n"
"\n"
"  \"The Program\" refers to any copyrightable work licensed under this\n"
"License.  Each licensee is addressed as \"you\".  \"Licensees\" and\n"
"\"recipients\" may be individuals or organizations.\n"
"\n"
"  To \"modify\" a work means to copy from or adapt all or part of the work\n"
"in a fashion requiring copyright permission, other than the making of an\n"
"exact copy.  The resulting work is called a \"modified version\" of the\n"
"earlier work or a work \"based on\" the earlier work.\n"
"\n"
"  A \"covered work\" means either the unmodified Program or a work based\n"
"on the Program.\n"
"\n"
"  To \"propagate\" a work means to do anything with it that, without\n"
"permission, would make you directly or secondarily liable for\n"
"infringement under applicable copyright law, except executing it on a\n"
"computer or modifying a private copy.  Propagation includes copying,\n"
"distribution (with or without modification), making available to the\n"
"public, and in some countries other activities as well.\n"
"\n"
"  To \"convey\" a work means any kind of propagation that enables other\n"
"parties to make or receive copies.  Mere interaction with a user through\n"
"a computer network, with no transfer of a copy, is not conveying.\n"
"\n"
"  An interactive user interface displays \"Appropriate Legal Notices\"\n"
"to the extent that it includes a convenient and prominently visible\n"
"feature that (1) displays an appropriate copyright notice, and (2)\n"
"tells the user that there is no warranty for the work (except to the\n"
"extent that warranties are provided), that licensees may convey the\n"
"work under this License, and how to view a copy of this License.  If\n"
"the interface presents a list of user commands or options, such as a\n"
"menu, a prominent item in the list meets this criterion.\n"
"\n"
"  1. Source Code.\n"
"\n"
"  The \"source code\" for a work means the preferred form of the work\n"
"for making modifications to it.  \"Object code\" means any non-source\n"
"form of a work.\n"
"\n"
"  A \"Standard Interface\" means an interface that either is an official\n"
"standard defined by a recognized standards body, or, in the case of\n"
"interfaces specified for a particular programming language, one that\n"
"is widely used among developers working in that language.\n"
"\n"
"  The \"System Libraries\" of an executable work include anything, other\n"
"than the work as a whole, that (a) is included in the normal form of\n"
"packaging a Major Component, but which is not part of that Major\n"
"Component, and (b) serves only to enable use of the work with that\n"
"Major Component, or to implement a Standard Interface for which an\n"
"implementation is available to the public in source code form.  A\n"
"\"Major Component\", in this context, means a major essential component\n"
"(kernel, window system, and so on) of the specific operating system\n"
"(if any) on which the executable work runs, or a compiler used to\n"
"produce the work, or an object code interpreter used to run it.\n"
"\n"
"  The \"Corresponding Source\" for a work in object code form means all\n"
"the source code needed to generate, install, and (for an executable\n"
"work) run the object code and to modify the work, including scripts to\n"
"control those activities.  However, it does not include the work\'s\n"
"System Libraries, or general-purpose tools or generally available free\n"
"programs which are used unmodified in performing those activities but\n"
"which are not part of the work.  For example, Corresponding Source\n"
"includes interface definition files associated with source files for\n"
"the work, and the source code for shared libraries and dynamically\n"
"linked subprograms that the work is specifically designed to require,\n"
"such as by intimate data communication or control flow between those\n"
"subprograms and other parts of the work.\n"
"\n"
"  The Corresponding Source need not include anything that users\n"
"can regenerate automatically from other parts of the Corresponding\n"
"Source.\n"
"\n"
"  The Corresponding Source for a work in source code form is that\n"
"same work.\n"
"\n"
"  2. Basic Permissions.\n"
"\n"
"  All rights granted under this License are granted for the term of\n"
"copyright on the Program, and are irrevocable provided the stated\n"
"conditions are met.  This License explicitly affirms your unlimited\n"
"permission to run the unmodified Program.  The output from running a\n"
"covered work is covered by this License only if the output, given its\n"
"content, constitutes a covered work.  This License acknowledges your\n"
"rights of fair use or other equivalent, as provided by copyright law.\n"
"\n"
"  You may make, run and propagate covered works that you do not\n"
"convey, without conditions so long as your license otherwise remains\n"
"in force.  You may convey covered works to others for the sole purpose\n"
"of having them make modifications exclusively for you, or provide you\n"
"with facilities for running those works, provided that you comply with\n"
"the terms of this License in conveying all material for which you do\n"
"not control copyright.  Those thus making or running the covered works\n"
"for you must do so exclusively on your behalf, under your direction\n"
"and control, on terms that prohibit them from making any copies of\n"
"your copyrighted material outside their relationship with you.\n"
"\n"
"  Conveying under any other circumstances is permitted solely under\n"
"the conditions stated below.  Sublicensing is not allowed; section 10\n"
"makes it unnecessary.\n"
"\n"
"  3. Protecting Users\' Legal Rights From Anti-Circumvention Law.\n"
"\n"
"  No covered work shall be deemed part of an effective technological\n"
"measure under any applicable law fulfilling obligations under article\n"
"11 of the WIPO copyright treaty adopted on 20 December 1996, or\n"
"similar laws prohibiting or restricting circumvention of such\n"
"measures.\n"
"\n"
"  When you convey a covered work, you waive any legal power to forbid\n"
"circumvention of technological measures to the extent such circumvention\n"
"is effected by exercising rights under this License with respect to\n"
"the covered work, and you disclaim any intention to limit operation or\n"
"modification of the work as a means of enforcing, against the work\'s\n"
"users, your or third parties\' legal rights to forbid circumvention of\n"
"technological measures.\n"
"\n"
"  4. Conveying Verbatim Copies.\n"
"\n"
"  You may convey verbatim copies of the Program\'s source code as you\n"
"receive it, in any medium, provided that you conspicuously and\n"
"appropriately publish on each copy an appropriate copyright notice;\n"
"keep intact all notices stating that this License and any\n"
"non-permissive terms added in accord with section 7 apply to the code;\n"
"keep intact all notices of the absence of any warranty; and give all\n"
"recipients a copy of this License along with the Program.\n"
"\n"
"  You may charge any price or no price for each copy that you convey,\n"
"and you may offer support or warranty protection for a fee.\n"
"\n"
"  5. Conveying Modified Source Versions.\n"
"\n"
"  You may convey a work based on the Program, or the modifications to\n"
"produce it from the Program, in the form of source code under the\n"
"terms of section 4, provided that you also meet all of these conditions:\n"
"\n"
"    a) The work must carry prominent notices stating that you modified\n"
"    it, and giving a relevant date.\n"
"\n"
"    b) The work must carry prominent notices stating that it is\n"
"    released under this License and any conditions added under section\n"
"    7.  This requirement modifies the requirement in section 4 to\n"
"    \"keep intact all notices\".\n"
"\n"
"    c) You must license the entire work, as a whole, under this\n"
"    License to anyone who comes into possession of a copy.  This\n"
"    License will therefore apply, along with any applicable section 7\n"
"    additional terms, to the whole of the work, and all its parts,\n"
"    regardless of how they are packaged.  This License gives no\n"
"    permission to license the work in any other way, but it does not\n"
"    invalidate such permission if you have separately received it.\n"
"\n"
"    d) If the work has interactive user interfaces, each must display\n"
"    Appropriate Legal Notices; however, if the Program has interactive\n"
"    interfaces that do not display Appropriate Legal Notices, your\n"
"    work need not make them do so.\n"
"\n"
"  A compilation of a covered work with other separate and independent\n"
"works, which are not by their nature extensions of the covered work,\n"
"and which are not combined with it such as to form a larger program,\n"
"in or on a volume of a storage or distribution medium, is called an\n"
"\"aggregate\" if the compilation and its resulting copyright are not\n"
"used to limit the access or legal rights of the compilation\'s users\n"
"beyond what the individual works permit.  Inclusion of a covered work\n"
"in an aggregate does not cause this License to apply to the other\n"
"parts of the aggregate.\n"
"\n"
"  6. Conveying Non-Source Forms.\n"
"\n"
"  You may convey a covered work in object code form under the terms\n"
"of sections 4 and 5, provided that you also convey the\n"
"machine-readable Corresponding Source under the terms of this License,\n"
"in one of these ways:\n"
"\n"
"    a) Convey the object code in, or embodied in, a physical product\n"
"    (including a physical distribution medium), accompanied by the\n"
"    Corresponding Source fixed on a durable physical medium\n"
"    customarily used for software interchange.\n"
"\n"
"    b) Convey the object code in, or embodied in, a physical product\n"
"    (including a physical distribution medium), accompanied by a\n"
"    written offer, valid for at least three years and valid for as\n"
"    long as you offer spare parts or customer support for that product\n"
"    model, to give anyone who possesses the object code either (1) a\n"
"    copy of the Corresponding Source for all the software in the\n"
"    product that is covered by this License, on a durable physical\n"
"    medium customarily used for software interchange, for a price no\n"
"    more than your reasonable cost of physically performing this\n"
"    conveying of source, or (2) access to copy the\n"
"    Corresponding Source from a network server at no charge.\n"
"\n"
"    c) Convey individual copies of the object code with a copy of the\n"
"    written offer to provide the Corresponding Source.  This\n"
"    alternative is allowed only occasionally and noncommercially, and\n"
"    only if you received the object code with such an offer, in accord\n"
"    with subsection 6b.\n"
"\n"
"    d) Convey the object code by offering access from a designated\n"
"    place (gratis or for a charge), and offer equivalent access to the\n"
"    Corresponding Source in the same way through the same place at no\n"
"    further charge.  You need not require recipients to copy the\n"
"    Corresponding Source along with the object code.  If the place to\n"
"    copy the object code is a network server, the Corresponding Source\n"
"    may be on a different server (operated by you or a third party)\n"
"    that supports equivalent copying facilities, provided you maintain\n"
"    clear directions next to the object code saying where to find the\n"
"    Corresponding Source.  Regardless of what server hosts the\n"
"    Corresponding Source, you remain obligated to ensure that it is\n"
"    available for as long as needed to satisfy these requirements.\n"
"\n"
"    e) Convey the object code using peer-to-peer transmission, provided\n"
"    you inform other peers where the object code and Corresponding\n"
"    Source of the work are being offered to the general public at no\n"
"    charge under subsection 6d.\n"
"\n"
"  A separable portion of the object code, whose source code is excluded\n"
"from the Corresponding Source as a System Library, need not be\n"
"included in conveying the object code work.\n"
"\n"
"  A \"User Product\" is either (1) a \"consumer product\", which means any\n"
"tangible personal property which is normally used for personal, family,\n"
"or household purposes, or (2) anything designed or sold for incorporation\n"
"into a dwelling.  In determining whether a product is a consumer product,\n"
"doubtful cases shall be resolved in favor of coverage.  For a particular\n"
"product received by a particular user, \"normally used\" refers to a\n"
"typical or common use of that class of product, regardless of the status\n"
"of the particular user or of the way in which the particular user\n"
"actually uses, or expects or is expected to use, the product.  A product\n"
"is a consumer product regardless of whether the product has substantial\n"
"commercial, industrial or non-consumer uses, unless such uses represent\n"
"the only significant mode of use of the product.\n"
"\n"
"  \"Installation Information\" for a User Product means any methods,\n"
"procedures, authorization keys, or other information required to install\n"
"and execute modified versions of a covered work in that User Product from\n"
"a modified version of its Corresponding Source.  The information must\n"
"suffice to ensure that the continued functioning of the modified object\n"
"code is in no case prevented or interfered with solely because\n"
"modification has been made.\n"
"\n"
"  If you convey an object code work under this section in, or with, or\n"
"specifically for use in, a User Product, and the conveying occurs as\n"
"part of a transaction in which the right of possession and use of the\n"
"User Product is transferred to the recipient in perpetuity or for a\n"
"fixed term (regardless of how the transaction is characterized), the\n"
"Corresponding Source conveyed under this section must be accompanied\n"
"by the Installation Information.  But this requirement does not apply\n"
"if neither you nor any third party retains the ability to install\n"
"modified object code on the User Product (for example, the work has\n"
"been installed in ROM).\n"
"\n"
"  The requirement to provide Installation Information does not include a\n"
"requirement to continue to provide support service, warranty, or updates\n"
"for a work that has been modified or installed by the recipient, or for\n"
"the User Product in which it has been modified or installed.  Access to a\n"
"network may be denied when the modification itself materially and\n"
"adversely affects the operation of the network or violates the rules and\n"
"protocols for communication across the network.\n"
"\n"
"  Corresponding Source conveyed, and Installation Information provided,\n"
"in accord with this section must be in a format that is publicly\n"
"documented (and with an implementation available to the public in\n"
"source code form), and must require no special password or key for\n"
"unpacking, reading or copying.\n"
"\n"
"  7. Additional Terms.\n"
"\n"
"  \"Additional permissions\" are terms that supplement the terms of this\n"
"License by making exceptions from one or more of its conditions.\n"
"Additional permissions that are applicable to the entire Program shall\n"
"be treated as though they were included in this License, to the extent\n"
"that they are valid under applicable law.  If additional permissions\n"
"apply only to part of the Program, that part may be used separately\n"
"under those permissions, but the entire Program remains governed by\n"
"this License without regard to the additional permissions.\n"
"\n"
"  When you convey a copy of a covered work, you may at your option\n"
"remove any additional permissions from that copy, or from any part of\n"
"it.  (Additional permissions may be written to require their own\n"
"removal in certain cases when you modify the work.)  You may place\n"
"additional permissions on material, added by you to a covered work,\n"
"for which you have or can give appropriate copyright permission.\n"
"\n"
"  Notwithstanding any other provision of this License, for material you\n"
"add to a covered work, you may (if authorized by the copyright holders of\n"
"that material) supplement the terms of this License with terms:\n"
"\n"
"    a) Disclaiming warranty or limiting liability differently from the\n"
"    terms of sections 15 and 16 of this License; or\n"
"\n"
"    b) Requiring preservation of specified reasonable legal notices or\n"
"    author attributions in that material or in the Appropriate Legal\n"
"    Notices displayed by works containing it; or\n"
"\n"
"    c) Prohibiting misrepresentation of the origin of that material, or\n"
"    requiring that modified versions of such material be marked in\n"
"    reasonable ways as different from the original version; or\n"
"\n"
"    d) Limiting the use for publicity purposes of names of licensors or\n"
"    authors of the material; or\n"
"\n"
"    e) Declining to grant rights under trademark law for use of some\n"
"    trade names, trademarks, or service marks; or\n"
"\n"
"    f) Requiring indemnification of licensors and authors of that\n"
"    material by anyone who conveys the material (or modified versions of\n"
"    it) with contractual assumptions of liability to the recipient, for\n"
"    any liability that these contractual assumptions directly impose on\n"
"    those licensors and authors.\n"
"\n"
"  All other non-permissive additional terms are considered \"further\n"
"restrictions\" within the meaning of section 10.  If the Program as you\n"
"received it, or any part of it, contains a notice stating that it is\n"
"governed by this License along with a term that is a further\n"
"restriction, you may remove that term.  If a license document contains\n"
"a further restriction but permits relicensing or conveying under this\n"
"License, you may add to a covered work material governed by the terms\n"
"of that license document, provided that the further restriction does\n"
"not survive such relicensing or conveying.\n"
"\n"
"  If you add terms to a covered work in accord with this section, you\n"
"must place, in the relevant source files, a statement of the\n"
"additional terms that apply to those files, or a notice indicating\n"
"where to find the applicable terms.\n"
"\n"
"  Additional terms, permissive or non-permissive, may be stated in the\n"
"form of a separately written license, or stated as exceptions;\n"
"the above requirements apply either way.\n"
"\n"
"  8. Termination.\n"
"\n"
"  You may not propagate or modify a covered work except as expressly\n"
"provided under this License.  Any attempt otherwise to propagate or\n"
"modify it is void, and will automatically terminate your rights under\n"
"this License (including any patent licenses granted under the third\n"
"paragraph of section 11).\n"
"\n"
"  However, if you cease all violation of this License, then your\n"
"license from a particular copyright holder is reinstated (a)\n"
"provisionally, unless and until the copyright holder explicitly and\n"
"finally terminates your license, and (b) permanently, if the copyright\n"
"holder fails to notify you of the violation by some reasonable means\n"
"prior to 60 days after the cessation.\n"
"\n"
"  Moreover, your license from a particular copyright holder is\n"
"reinstated permanently if the copyright holder notifies you of the\n"
"violation by some reasonable means, this is the first time you have\n"
"received notice of violation of this License (for any work) from that\n"
"copyright holder, and you cure the violation prior to 30 days after\n"
"your receipt of the notice.\n"
"\n"
"  Termination of your rights under this section does not terminate the\n"
"licenses of parties who have received copies or rights from you under\n"
"this License.  If your rights have been terminated and not permanently\n"
"reinstated, you do not qualify to receive new licenses for the same\n"
"material under section 10.\n"
"\n"
"  9. Acceptance Not Required for Having Copies.\n"
"\n"
"  You are not required to accept this License in order to receive or\n"
"run a copy of the Program.  Ancillary propagation of a covered work\n"
"occurring solely as a consequence of using peer-to-peer transmission\n"
"to receive a copy likewise does not require acceptance.  However,\n"
"nothing other than this License grants you permission to propagate or\n"
"modify any covered work.  These actions infringe copyright if you do\n"
"not accept this License.  Therefore, by modifying or propagating a\n"
"covered work, you indicate your acceptance of this License to do so.\n"
"\n"
"  10. Automatic Licensing of Downstream Recipients.\n"
"\n"
"  Each time you convey a covered work, the recipient automatically\n"
"receives a license from the original licensors, to run, modify and\n"
"propagate that work, subject to this License.  You are not responsible\n"
"for enforcing compliance by third parties with this License.\n"
"\n"
"  An \"entity transaction\" is a transaction transferring control of an\n"
"organization, or substantially all assets of one, or subdividing an\n"
"organization, or merging organizations.  If propagation of a covered\n"
"work results from an entity transaction, each party to that\n"
"transaction who receives a copy of the work also receives whatever\n"
"licenses to the work the party\'s predecessor in interest had or could\n"
"give under the previous paragraph, plus a right to possession of the\n"
"Corresponding Source of the work from the predecessor in interest, if\n"
"the predecessor has it or can get it with reasonable efforts.\n"
"\n"
"  You may not impose any further restrictions on the exercise of the\n"
"rights granted or affirmed under this License.  For example, you may\n"
"not impose a license fee, royalty, or other charge for exercise of\n"
"rights granted under this License, and you may not initiate litigation\n"
"(including a cross-claim or counterclaim in a lawsuit) alleging that\n"
"any patent claim is infringed by making, using, selling, offering for\n"
"sale, or importing the Program or any portion of it.\n"
"\n"
"  11. Patents.\n"
"\n"
"  A \"contributor\" is a copyright holder who authorizes use under this\n"
"License of the Program or a work on which the Program is based.  The\n"
"work thus licensed is called the contributor\'s \"contributor version\".\n"
"\n"
"  A contributor\'s \"essential patent claims\" are all patent claims\n"
"owned or controlled by the contributor, whether already acquired or\n"
"hereafter acquired, that would be infringed by some manner, permitted\n"
"by this License, of making, using, or selling its contributor version,\n"
"but do not include claims that would be infringed only as a\n"
"consequence of further modification of the contributor version.  For\n"
"purposes of this definition, \"control\" includes the right to grant\n"
"patent sublicenses in a manner consistent with the requirements of\n"
"this License.\n"
"\n"
"  Each contributor grants you a non-exclusive, worldwide, royalty-free\n"
"patent license under the contributor\'s essential patent claims, to\n"
"make, use, sell, offer for sale, import and otherwise run, modify and\n"
"propagate the contents of its contributor version.\n"
"\n"
"  In the following three paragraphs, a \"patent license\" is any express\n"
"agreement or commitment, however denominated, not to enforce a patent\n"
"(such as an express permission to practice a patent or covenant not to\n"
"sue for patent infringement).  To \"grant\" such a patent license to a\n"
"party means to make such an agreement or commitment not to enforce a\n"
"patent against the party.\n"
"\n"
"  If you convey a covered work, knowingly relying on a patent license,\n"
"and the Corresponding Source of the work is not available for anyone\n"
"to copy, free of charge and under the terms of this License, through a\n"
"publicly available network server or other readily accessible means,\n"
"then you must either (1) cause the Corresponding Source to be so\n"
"available, or (2) arrange to deprive yourself of the benefit of the\n"
"patent license for this particular work, or (3) arrange, in a manner\n"
"consistent with the requirements of this License, to extend the patent\n"
"license to downstream recipients.  \"Knowingly relying\" means you have\n"
"actual knowledge that, but for the patent license, your conveying the\n"
"covered work in a country, or your recipient\'s use of the covered work\n"
"in a country, would infringe one or more identifiable patents in that\n"
"country that you have reason to believe are valid.\n"
"\n"
"  If, pursuant to or in connection with a single transaction or\n"
"arrangement, you convey, or propagate by procuring conveyance of, a\n"
"covered work, and grant a patent license to some of the parties\n"
"receiving the covered work authorizing them to use, propagate, modify\n"
"or convey a specific copy of the covered work, then the patent license\n"
"you grant is automatically extended to all recipients of the covered\n"
"work and works based on it.\n"
"\n"
"  A patent license is \"discriminatory\" if it does not include within\n"
"the scope of its coverage, prohibits the exercise of, or is\n"
"conditioned on the non-exercise of one or more of the rights that are\n"
"specifically granted under this License.  You may not convey a covered\n"
"work if you are a party to an arrangement with a third party that is\n"
"in the business of distributing software, under which you make payment\n"
"to the third party based on the extent of your activity of conveying\n"
"the work, and under which the third party grants, to any of the\n"
"parties who would receive the covered work from you, a discriminatory\n"
"patent license (a) in connection with copies of the covered work\n"
"conveyed by you (or copies made from those copies), or (b) primarily\n"
"for and in connection with specific products or compilations that\n"
"contain the covered work, unless you entered into that arrangement,\n"
"or that patent license was granted, prior to 28 March 2007.\n"
"\n"
"  Nothing in this License shall be construed as excluding or limiting\n"
"any implied license or other defenses to infringement that may\n"
"otherwise be available to you under applicable patent law.\n"
"\n"
"  12. No Surrender of Others\' Freedom.\n"
"\n"
"  If conditions are imposed on you (whether by court order, agreement or\n"
"otherwise) that contradict the conditions of this License, they do not\n"
"excuse you from the conditions of this License.  If you cannot convey a\n"
"covered work so as to satisfy simultaneously your obligations under this\n"
"License and any other pertinent obligations, then as a consequence you may\n"
"not convey it at all.  For example, if you agree to terms that obligate you\n"
"to collect a royalty for further conveying from those to whom you convey\n"
"the Program, the only way you could satisfy both those terms and this\n"
"License would be to refrain entirely from conveying the Program.\n"
"\n"
"  13. Use with the GNU Affero General Public License.\n"
"\n"
"  Notwithstanding any other provision of this License, you have\n"
"permission to link or combine any covered work with a work licensed\n"
"under version 3 of the GNU Affero General Public License into a single\n"
"combined work, and to convey the resulting work.  The terms of this\n"
"License will continue to apply to the part which is the covered work,\n"
"but the special requirements of the GNU Affero General Public License,\n"
"section 13, concerning interaction through a network will apply to the\n"
"combination as such.\n"
"\n"
"  14. Revised Versions of this License.\n"
"\n"
"  The Free Software Foundation may publish revised and/or new versions of\n"
"the GNU General Public License from time to time.  Such new versions will\n"
"be similar in spirit to the present version, but may differ in detail to\n"
"address new problems or concerns.\n"
"\n"
"  Each version is given a distinguishing version number.  If the\n"
"Program specifies that a certain numbered version of the GNU General\n"
"Public License \"or any later version\" applies to it, you have the\n"
"option of following the terms and conditions either of that numbered\n"
"version or of any later version published by the Free Software\n"
"Foundation.  If the Program does not specify a version number of the\n"
"GNU General Public License, you may choose any version ever published\n"
"by the Free Software Foundation.\n"
"\n"
"  If the Program specifies that a proxy can decide which future\n"
"versions of the GNU General Public License can be used, that proxy\'s\n"
"public statement of acceptance of a version permanently authorizes you\n"
"to choose that version for the Program.\n"
"\n"
"  Later license versions may give you additional or different\n"
"permissions.  However, no additional obligations are imposed on any\n"
"author or copyright holder as a result of your choosing to follow a\n"
"later version.\n"
"\n"
"  15. Disclaimer of Warranty.\n"
"\n"
"  THERE IS NO WARRANTY FOR THE PROGRAM, TO THE EXTENT PERMITTED BY\n"
"APPLICABLE LAW.  EXCEPT WHEN OTHERWISE STATED IN WRITING THE COPYRIGHT\n"
"HOLDERS AND/OR OTHER PARTIES PROVIDE THE PROGRAM \"AS IS\" WITHOUT WARRANTY\n"
"OF ANY KIND, EITHER EXPRESSED OR IMPLIED, INCLUDING, BUT NOT LIMITED TO,\n"
"THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR\n"
"PURPOSE.  THE ENTIRE RISK AS TO THE QUALITY AND PERFORMANCE OF THE PROGRAM\n"
"IS WITH YOU.  SHOULD THE PROGRAM PROVE DEFECTIVE, YOU ASSUME THE COST OF\n"
"ALL NECESSARY SERVICING, REPAIR OR CORRECTION.\n"
"\n"
"  16. Limitation of Liability.\n"
"\n"
"  IN NO EVENT UNLESS REQUIRED BY APPLICABLE LAW OR AGREED TO IN WRITING\n"
"WILL ANY COPYRIGHT HOLDER, OR ANY OTHER PARTY WHO MODIFIES AND/OR CONVEYS\n"
"THE PROGRAM AS PERMITTED ABOVE, BE LIABLE TO YOU FOR DAMAGES, INCLUDING ANY\n"
"GENERAL, SPECIAL, INCIDENTAL OR CONSEQUENTIAL DAMAGES ARISING OUT OF THE\n"
"USE OR INABILITY TO USE THE PROGRAM (INCLUDING BUT NOT LIMITED TO LOSS OF\n"
"DATA OR DATA BEING RENDERED INACCURATE OR LOSSES SUSTAINED BY YOU OR THIRD\n"
"PARTIES OR A FAILURE OF THE PROGRAM TO OPERATE WITH ANY OTHER PROGRAMS),\n"
"EVEN IF SUCH HOLDER OR OTHER PARTY HAS BEEN ADVISED OF THE POSSIBILITY OF\n"
"SUCH DAMAGES.\n"
"\n"
"  17. Interpretation of Sections 15 and 16.\n"
"\n"
"  If the disclaimer of warranty and limitation of liability provided\n"
"above cannot be given local legal effect according to their terms,\n"
"reviewing courts shall apply local law that most closely approximates\n"
"an absolute waiver of all civil liability in connection with the\n"
"Program, unless a warranty or assumption of liability accompanies a\n"
"copy of the Program in return for a fee."))



"""Start the lifescale GUI from here!

Copyright (C) 2022  Andreas Hellerschmied <heller182@gmx.at>
"""
from lifescale.gui.gui_main import main


def run_gui():
    """Start the GUI."""
    main()


if __name__ == "__main__":
    """Main Program."""
    run_gui()



"""Sample model class for PyQt's model view architectrue.

Copyright (C) 2022  Andreas Hellerschmied <heller182@gmx.at>

This program is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program.  If not, see <https://www.gnu.org/licenses/>.
"""

from PyQt6.QtCore import QAbstractTableModel, Qt
from PyQt6 import QtGui


NONE_REPRESENTATION_IN_TABLE_VIEW = ''  # Representation of None values in table views in the GUI


class SampleSummaryModel(QAbstractTableModel):
    """Model for the panel metadata table view."""

    # Number of decimal pla
    _DECIMAL_PLACES_PER_FLOAT_COLUMN = {
        'Concentration': 1,
        'StartTime': 3,
        'EndTime': 3,
        'MeanMass': 3,
        'MassContent': 5,
        'MeasuredVolume': 3,
        'MeasuredVolume': 3,
        'ResonantFrequency': 3,
        'Mass_median': 3,
        'Mass_std': 3,
        'Mass_q25': 3,
        'Mass_q75': 3,
        'Mass_iqr': 3,
        'keep_max_mass': 3,
        'keep_min_mass': 3,
        'DetectedParticles_keep': 0,
        'Mass_mean_keep': 3,
        'Mass_median_keep': 3,
        'Mass_std_keep': 3,
        'Mass_q25_keep': 3,
        'Mass_q75_keep': 3,
        'Mass_iqr_keep': 3,
        'Mass_rCV_keep': 3,
        'Mass_rCV': 3,
    }

    def __init__(self, df):
        QAbstractTableModel.__init__(self)
        self._data = None
        self.load_data(df)

    def load_data(self, df):
        """Load data from pandas dataframe to table model."""
        self._data = df

    def rowCount(self, parent=None):
        return self._data.shape[0]

    def columnCount(self, parent=None):
        return self._data.shape[1]

    def data(self, index, role=Qt.ItemDataRole.DisplayRole):
        if index.isValid():

            value = self._data.iloc[index.row(), index.column()]
            column_name = self.get_colum_names[index.column()]

            if role == Qt.ItemDataRole.DisplayRole:

                # Custom formatter (string is expected as return type):
                if value is None:  #
                    return NONE_REPRESENTATION_IN_TABLE_VIEW
                elif isinstance(value, float):
                    if value != value:  # True, if value is "NaN"
                        return NONE_REPRESENTATION_IN_TABLE_VIEW
                    else:
                        if column_name in self._DECIMAL_PLACES_PER_FLOAT_COLUMN.keys():
                            num_dec_places = self._DECIMAL_PLACES_PER_FLOAT_COLUMN[column_name]
                            return '{1:.{0}f}'.format(num_dec_places, value)
                        else:
                            return str(value)
                else:  # all other
                    return str(value)

            if role == Qt.ItemDataRole.TextAlignmentRole:
                # value = self._data.iloc[index.row(), index.column()]
                if isinstance(value, int) or isinstance(value, float):
                    # Align right, vertical middle.
                    return Qt.AlignmentFlag.AlignVCenter + Qt.AlignmentFlag.AlignRight

            if role == Qt.ItemDataRole.BackgroundRole:
                if (self._data.iloc[index.row()].keep_max_mass is not None) or (self._data.iloc[index.row()].keep_min_mass is not None):
                    return QtGui.QColor('green')

            # if role == Qt.BackgroundRole:
            #     if column_name == 'is_datum':
            #         if value:
            #             return QtGui.QColor('red')
            #
            #     is_observed_flag = self._data.iloc[index.row(), 7]  # is_observed
            #     if is_observed_flag:
            #         return QtGui.QColor('cyan')

            # if role == Qt.CheckStateRole:
            #     try:
            #         if column_name == 'is_datum':
            #             keep_obs_flag = self._data.iloc[index.row(), self._data_column_names.index('is_datum')]
            #             if keep_obs_flag:
            #                 return Qt.Checked
            #             else:
            #                 return Qt.Unchecked
            #     except Exception:
            #         return None
        return None

    def headerData(self, section, orientation, role):
        # section is the index of the column/row.
        if role == Qt.ItemDataRole.DisplayRole:
            if self._data is not None:
                if orientation == Qt.Orientation.Horizontal:
                    # return self._SHOW_COLUMNS_IN_TABLE_DICT[str(self._data.columns[section])]
                    return str(self._data.columns[section])
                if orientation == Qt.Orientation.Vertical:
                    return str(self._data.index[section])

    def flags(self, index):
        """Enable editing of table items."""
        flags = super(self.__class__, self).flags(index)
        flags |= Qt.ItemFlag.ItemIsSelectable
        flags |= Qt.ItemFlag.ItemIsEnabled
        flags |= Qt.ItemFlag.ItemIsDragEnabled
        flags |= Qt.ItemFlag.ItemIsDropEnabled
        return flags

    @property
    def get_data(self):
        return self._data

    @property
    def get_colum_names(self):
        """Return a list with all columns."""
        if self._data is not None:
            return self._data.columns.to_list()
        else:
            return None

    @property
    def get_sample_ids(self):
        """Return a list with all sample IDs."""
        if self._data is not None:
            return self._data['SampleID'].to_list()
        else:
            return None

    @property
    def get_sample_list_tags(self):
        """Return a list of sample tags with sample IDs and number of detected particles."""
        if self._data is not None:
            return (self._data['SampleID'].astype(str) + ': ' + self._data['DetectedParticles'].astype(str)).to_list()
        else:
            return None





"""Masses model class for PyQt's model view architectrue.

Copyright (C) 2022  Andreas Hellerschmied <heller182@gmx.at>

This program is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program.  If not, see <https://www.gnu.org/licenses/>.
"""

from PyQt6.QtCore import QAbstractTableModel, Qt
from PyQt6.QtWidgets import QMessageBox
from PyQt6 import QtGui


NONE_REPRESENTATION_IN_TABLE_VIEW = ''  # Representation of None values in table views in the GUI


class MassesModel(QAbstractTableModel):
    """Model for the masses table view."""

    # Number of decimal pla
    _DECIMAL_PLACES_PER_FLOAT_COLUMN = {
        'Time': 3,
        'Mass': 3,
        'Volume': 3,
        'TransitTime': 3,
        'PressureDrop': 5,
    }

    def __init__(self, df):
        QAbstractTableModel.__init__(self)
        self._data = None
        self._masses = None
        self._current_sample_id = None
        self.load_data(df)

    def load_data(self, df):
        """Load data from pandas dataframe to table model.

        Notes
        -----
        The data is assigned by reference, i.e. all changes in `_masses` will propagate to the data origin (`df`).
        """
        # self._masses = df.copy(deep=True)
        self._masses = df

    def update_view_model(self, sample_id: int):
        """Update the `_data` DataFrame that hold the actual data that is displayed."""
        try:
            df_masses_slice = self._masses.loc[self._masses['SampleID'] == sample_id]
        except KeyError:
            self._current_sample_id = None
            self._data = None
            QMessageBox.critical(self.parent(), 'Error!', f'SampleID "{sample_id}" is not available.')
        else:
            self._current_sample_id = sample_id
            self._data = df_masses_slice

    def rowCount(self, parent=None):
        if self._data is not None:
            return self._data.shape[0]
        else:
            return 0

    def columnCount(self, parent=None):
        if self._data is not None:
            return self._data.shape[1]
        else:
            return 0

    def data(self, index, role=Qt.ItemDataRole.DisplayRole):
        if index.isValid():

            value = self._data.iloc[index.row(), index.column()]
            column_name = self.get_colum_names[index.column()]

            if role == Qt.ItemDataRole.DisplayRole:

                # Custom formatter (string is expected as return type):
                if value is None:  #
                    return NONE_REPRESENTATION_IN_TABLE_VIEW
                elif isinstance(value, float):
                    if value != value:  # True, if value is "NaN"
                        return NONE_REPRESENTATION_IN_TABLE_VIEW
                    else:
                        if column_name in self._DECIMAL_PLACES_PER_FLOAT_COLUMN.keys():
                            num_dec_places = self._DECIMAL_PLACES_PER_FLOAT_COLUMN[column_name]
                            return '{1:.{0}f}'.format(num_dec_places, value)
                        else:
                            return str(value)
                else:  # all other
                    return str(value)

            if role == Qt.ItemDataRole.TextAlignmentRole:
                # value = self._data.iloc[index.row(), index.column()]
                if isinstance(value, int) or isinstance(value, float):
                    # Align right, vertical middle.
                    return Qt.AlignmentFlag.AlignVCenter + Qt.AlignmentFlag.AlignRight

            if role == Qt.ItemDataRole.BackgroundRole:
                if ~self._data.iloc[index.row()].keep:
                    return QtGui.QColor('red')

            # if role == Qt.CheckStateRole:
            #     try:
            #         if column_name == 'is_datum':
            #             keep_obs_flag = self._data.iloc[index.row(), self._data_column_names.index('is_datum')]
            #             if keep_obs_flag:
            #                 return Qt.Checked
            #             else:
            #                 return Qt.Unchecked
            #     except Exception:
            #         return None
        return None

    def headerData(self, section, orientation, role):
        # section is the index of the column/row.
        if role == Qt.ItemDataRole.DisplayRole:
            if self._data is not None:
                if orientation == Qt.Orientation.Horizontal:
                    # return self._SHOW_COLUMNS_IN_TABLE_DICT[str(self._data.columns[section])]
                    return str(self._data.columns[section])
                if orientation == Qt.Orientation.Vertical:
                    return str(self._data.index[section])

    def flags(self, index):
        """Enable editing of table items."""
        flags = super(self.__class__, self).flags(index)
        flags |= Qt.ItemFlag.ItemIsSelectable
        flags |= Qt.ItemFlag.ItemIsEnabled
        flags |= Qt.ItemFlag.ItemIsDragEnabled
        flags |= Qt.ItemFlag.ItemIsDropEnabled
        return flags

    @property
    def get_data(self):
        return self._data

    @property
    def get_colum_names(self):
        """Return a list with all columns."""
        if self._data is not None:
            return self._data.columns.to_list()
        else:
            return None

    @property
    def get_current_sample_id(self):
        """Return the current sample ID."""
        return self._current_sample_id

